@@ -9,13 +9,13 @@ import sys
 import os
 import dbus
 import urllib2
-import urllib
 import httplib
-import conic
 import time
 import socket
 import array
+import subprocess
 
+import dbus
 from dbus.mainloop.glib import DBusGMainLoop
 
 from mms import message
@@ -27,8 +27,6 @@ import controller as fMMSController
 import logging
 log = logging.getLogger('fmms.%s' % __name__)
 
-magic = 0xacdcacdc
-
 _DBG = True
 
 class PushHandler:
@@ -120,12 +118,52 @@ class PushHandler:
 			interface.Notify("MMS", 0, '', message, sender, choices, {"category": "email-message", "dialog-type": 4, "led-pattern": "PatternCommunicationEmail", "dbus-callback-default": "se.frals.fmms /se/frals/fmms se.frals.fmms open_mms string:\"" + path + "\""}, -1)
 
 
+	def _get_mms_message(self, location, transaction):
+		if (self.config.get_experimental() == 1):
+			log.info("RUNNING IN EXPERIMENTAL MODE")
+
+			(proxyurl, proxyport) = self.config.get_proxy_from_apn()
+			apn = self.config.get_apn_from_osso()
+			mmsc1 = self.cont.get_host_from_url(self.config.get_mmsc())
+			mmsc2 = self.cont.get_host_from_url(location)
+			# todo get user/pass from gconf
+			try:
+				connector = ConnectionHandler(apn, "", "", proxyurl, mmsc1, mmsc2)
+				connector.start()
+			except:
+				log.exception("Connection failed.")
+		try:
+			dirname = self.__get_mms_message(location, transaction)
+		except Exception, e:
+			log.exception("Something went wrong with getting the message... bailing out")
+			raise
+		
+		# send acknowledge we got it ok
+		try:
+			socket.setdefaulttimeout(20)
+			ack = self._send_acknowledge(transaction)
+			log.info("ack sent")
+		except:
+			log.exception("sending ack failed")
+		
+		if (self.config.get_experimental() == 1) and connector != None:
+			try:
+				connector.disconnect()
+			except:
+				log.exception("Failed to close connection.")
+		
+		return dirname
+		
+			
+
+
 	""" get the mms message from content-location """
 	""" thanks benaranguren on talk.maemo.org for patch including x-wap-profile header """
-	def _get_mms_message(self, location, transaction):
+	def __get_mms_message(self, location, transaction):
 		log.info("getting file: %s", location)
 		try:
-
+			(proxyurl, proxyport) = self.config.get_proxy_from_apn()
+			
 			try:
 				socket.setdefaulttimeout(20)
 				notifyresp = self._send_notify_resp(transaction)
@@ -136,7 +174,6 @@ class PushHandler:
 			# TODO: configurable time-out?
 			timeout = 30
 			socket.setdefaulttimeout(timeout)
-			(proxyurl, proxyport) = self.config.get_proxy_from_apn()
 			
 			if proxyurl == "" or proxyurl == None:
 				log.info("connecting without proxy")
@@ -161,19 +198,8 @@ class PushHandler:
 			mmsdataall = mmsdata.read()
 			dirname = self.cont.save_binary_mms(mmsdataall, transaction)
 			
-			if(_DBG):
-				log.info("fetched %s and wrote to file", location)
-				
-			# send acknowledge we got it ok
-			try:
-				socket.setdefaulttimeout(20)
-				ack = self._send_acknowledge(transaction)
-				log.info("ack sent")
-			except:
-				log.exception("sending ack failed: %s %s", type(e), e)
+			log.info("fetched %s and wrote to file", location)
 			
-			socket.setdefaulttimeout(timeout)
-
 
 		except Exception, e:
 			log.exception("fatal: %s %s", type(e), e)
@@ -181,8 +207,8 @@ class PushHandler:
 			proxy = bus.get_object('org.freedesktop.Notifications', '/org/freedesktop/Notifications')
 			interface = dbus.Interface(proxy,dbus_interface='org.freedesktop.Notifications')
 			interface.SystemNoteInfoprint ("fMMS: Failed to download MMS message.")
-			raise
-			
+			#raise
+		
 		return dirname
 
 
@@ -212,14 +238,16 @@ class PushHandler:
 		log.info("m-acknowledge-ind: %s", out)
 		return out
 
-  	    
+
 """ class for sending an mms """    	    
 class MMSSender:
-	def __init__(self, number=None, subject=None, message=None, attachment=None, sender=None, customMMS=None):
+	def __init__(self, number=None, subject=None, message=None, attachment=None, sender=None, customMMS=None, setupConn=False):
 		print "GOT SENDER:", sender
 		print "customMMS:", customMMS
 		self.customMMS = customMMS
 		self.config = fMMSconf.fMMS_config()
+		self.cont = fMMSController.fMMS_controller()
+		self.setupConn = setupConn
 		if customMMS == None:
 			self.number = number
 			self.subject = subject
@@ -228,6 +256,20 @@ class MMSSender:
 			self._mms = None
 			self._sender = sender
 			self.createMMS()
+			if self.setupConn == True and (self.config.get_experimental() == 1):
+				log.info("RUNNING SENDER IN EXPERIMENTAL MODE")
+		
+				(proxyurl, proxyport) = self.config.get_proxy_from_apn()
+				apn = self.config.get_apn_from_osso()
+				mmsc1 = self.cont.get_host_from_url(self.config.get_mmsc())
+				mmsc2 = 0
+				# todo get user/pass from gconf
+				try:
+					self.connector = ConnectionHandler(apn, "", "", proxyurl, mmsc1, mmsc2)
+					self.connector.start()
+				except:
+					log.exception("Connector failed")	
+				
 	    
 	def createMMS(self):
 		slide = message.MMSMessagePage()
@@ -298,4 +340,53 @@ class MMSSender:
 			outparsed = out
 			
 		log.info("MMSC RESPONDED: %s", outparsed)
-		return res.status, res.reason, outparsed, parsed
\ No newline at end of file
+
+		if (self.config.get_experimental() == 1) and self.setupConn == True:
+			try:
+				self.connector.disconnect()
+			except:
+				log.exception("Failed to close connection.")
+		
+		return res.status, res.reason, outparsed, parsed
+
+
+# TODO: error handling on this whole class, really.
+class ConnectionHandler:
+	
+	def __init__(self, apn, username="", password="", proxy="0", mmsc1="0", mmsc2="0"):
+		self.apn = apn
+		self.username = username
+		self.password = password
+		self.proxyip = proxy
+		self.mmsc1 = mmsc1
+		self.mmsc2 = mmsc2
+		self.rx = 0
+		self.tx = 0
+		log.info("ConnectionHandler UP!\nAPN: %s user: %s pass: %s proxyip: %s mmsc1: %s mmsc2: %s" % (self.apn, self.username, self.password, self.proxyip, self.mmsc1, self.mmsc2))
+		self.conn = self.connect()
+
+	def start(self):
+		args = "START %s %s %s %s %s %s" % (self.iface, self.ipaddr, self.mmsc1, self.mmsc2, self.dnsip, self.proxyip)
+		retcode = subprocess.call(["/opt/fmms/fmms_magic", args])
+		log.info("fmms_magic retcode: %s" % retcode)
+		
+	def connect(self):
+		bus = dbus.SystemBus()
+		gprs = dbus.Interface(bus.get_object("com.nokia.csd", "/com/nokia/csd/gprs"), "com.nokia.csd.GPRS")
+		obj = gprs.QuickConnect(self.apn, "IP", self.username, self.password)
+		conn = dbus.Interface(bus.get_object("com.nokia.csd", obj), "com.nokia.csd.GPRS.Context")
+		(apn, ctype, self.iface, self.ipaddr, connected, tx, rx) = conn.GetStatus()
+		
+		tmp = bus.get_object("com.nokia.csd.GPRS", obj)
+		props = dbus.Interface(tmp, "org.freedesktop.DBus.Properties")
+		self.dnsip = props.Get("com.nokia.csd.GPRS.Context", "PDNSAddress")
+		
+		return conn
+		
+	def disconnect(self):
+		log.info("running disconnect")
+		(apn, ctype, self.iface, self.ipaddr, connected, self.tx, self.rx) = self.conn.GetStatus()
+		args = "STOP %s" % self.iface
+		retcode = subprocess.call(["/opt/fmms/fmms_magic", args])
+		log.info("disconnecting connection. rx: %s tx: %s" % (self.tx, self.rx))
+		self.conn.Disconnect()
\ No newline at end of file
